#!/usr/bin/env node
'use strict';

var license = require('./license-66aabde9.js');
var fs = require('fs');
var sufNode = require('suf-node');
var fetch = require('node-fetch');
var sufLog = require('suf-log');
require('path');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var fetch__default = /*#__PURE__*/_interopDefaultLegacy(fetch);

async function getPackageJson() {
    const hasPackage = await sufNode.Exits('./package.json');
    if (!hasPackage) {
        license.log('info', 'No package.json found!');
        process.exit(1);
    }
    return JSON.parse((await fs.promises.readFile('./package.json')).toString());
}
function getArgs() {
    const [, , ...args] = process.argv;
    let ARGS = [];
    args.forEach((arg, i)=>{
        ARGS[i] = arg.toLowerCase();
    });
    return ARGS;
}

const configPath = (extension)=>`./suf.config.${extension}`;
async function checkConfigExists() {
    /*istanbul ignore else */ if (await sufNode.Exits(configPath('json'))) return 'json';
    /*istanbul ignore else */ if (await sufNode.Exits(configPath('ts'))) return 'ts';
    return null;
}
async function getConfig(Package) {
    const extension = await checkConfigExists();
    if (!extension) {
        license.log('info', 'No config Found, do you want to create one? ', '[Y/n]');
        const answer = await sufNode.getYNAnswer();
        if (answer) {
            const newConfig = await createOrUpdateConfig(Package);
            if (!newConfig) {
                process.exit(0);
            }
            return {
                config: newConfig,
                configPath: configPath('json')
            };
        }
        process.exit(1);
    } else {
        switch(extension){
            case 'json':
                return {
                    config: JSON.parse((await fs.promises.readFile(configPath('json'))).toString()),
                    configPath: configPath('json')
                };
            case 'ts':
                try {
                    const transpileModule = (await Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespace(require('typescript')); })).transpileModule;
                    const text = (await fs.promises.readFile(configPath('ts'))).toString();
                    const funcText = transpileModule(text, {}).outputText.replace(`Object.defineProperty(exports, "__esModule", { value: true });`, "const exports = {default: {}}") + "\nreturn exports.default";
                    const config = Function(funcText)();
                    if (!config) {
                        license.log('error', `${configPath('ts')} export is invalid.`);
                        process.exit(1);
                    }
                    return {
                        config: config || {},
                        configPath: configPath('ts')
                    };
                } catch (e) {
                    console.log(e);
                    license.log('error', 'Please install typescript or use a json config file!');
                    process.exit(1);
                }
        }
    }
}
async function createOrUpdateConfig(packageJson, type, STATE) {
    const path = STATE?.configPath || configPath('ts');
    if (!path.endsWith('.ts')) return undefined;
    let config;
    let updateOrCreate = 'Update';
    if (type && STATE) {
        license.log('info', `Adding ${type} to Config.`);
        config = {
            ...STATE.CONFIG,
            [type]: await configFuncs[type](packageJson)
        };
    } else {
        /**this will only be executed when there is no config. */ config = {};
        updateOrCreate = 'Create';
        for(const key in configFuncs){
            if (configFuncs.hasOwnProperty(key)) {
                const configFunc = configFuncs[key];
                license.log('info', `Add ${key} to Config?`, '[Y/n]');
                if (await sufNode.getYNAnswer()) {
                    config = {
                        ...config,
                        [key]: await configFunc(packageJson)
                    };
                }
            }
        }
    }
    console.log(`${sufLog.styler(`${updateOrCreate} Config`, license.colors.blue)}: ${path}`);
    fs.promises.writeFile(path, `import { ConfigFile } from 'suf-cli'

const config: ConfigFile = ${JSON.stringify(config, null, 2).replace(/^( *)"(.*)":/gm, '$1$2:')}

export default config`);
    return config;
}
const configFuncs = {
    badges: async (Package)=>{
        return {
            name: await getInp('Name', 'PACKAGE NAME', Package.name || ''),
            github: await getInp('Github', 'Username'),
            repo: await getInp('Github', 'Repo', Package.name || ''),
            out: await getInp('out', 'OUTPUT PATH', 'README.md'),
            badges: [
                ...await getBadges()
            ]
        };
    },
    tsDoc: async ()=>{
        return {
            title: await getInp('title', 'TITLE', 'Docs'),
            dir: await getInp('dir', 'INPUT PATH', 'dist'),
            out: await getInp('out', 'OUTPUT PATH', 'README.md')
        };
    },
    license: async ()=>{
        return {
            name: await getInp('name', 'FULL NAME(S)'),
            type: await getLicenseType(),
            year: await getInp('year', 'YEAR', new Date().getFullYear().toString()),
            out: await getInp('out', 'OUTPUT PATH', 'README.md'),
            file: await getInp('file', 'LICENSE FILE PATH', 'LICENSE')
        };
    }
};
async function getLicenseType() {
    process.stdout.write(sufLog.styler('Getting Licenses from api...\n', license.colors.gray));
    const licenses = await (await fetch__default["default"]('https://api.github.com/licenses')).json();
    process.stdout.write(sufLog.styler('Choose From: ' + JSON.stringify(licenses.map((l)=>l.key), null, 2).replace(/[\[\]",]/g, ''), license.colors.gray));
    return await getInp('type', 'LICENSE', 'mit');
}
async function getInp(type, type2, _default) {
    process.stdout.write(sufLog.styler(type, license.colors.blue));
    process.stdout.write(sufLog.styler(` ${type2}`, license.colors.yellow));
    process.stdout.write(sufLog.styler(': ', license.colors.gray));
    if (_default) {
        process.stdout.write(sufLog.styler(`(${_default})`, license.colors.gray));
        const input = await sufNode.readConsole();
        if (input.length === 0) {
            return _default;
        }
        return input;
    }
    return sufNode.readConsole();
}
async function getBadges(out) {
    if (!out) {
        out = [];
        process.stdout.write(sufLog.styler('Press Enter to Accept. ', license.colors.info));
        process.stdout.write(sufLog.styler('Format: ', license.colors.gray));
        process.stdout.write(sufLog.styler('BADGE ', license.colors.blue));
        process.stdout.write(sufLog.styler('LINK ', license.colors.yellow));
        process.stdout.write(sufLog.styler('Parameters?\n', license.colors.gray));
    }
    process.stdout.write(sufLog.styler('Add', license.colors.blue));
    process.stdout.write(sufLog.styler(' Badge', license.colors.yellow));
    process.stdout.write(sufLog.styler(': ', license.colors.gray));
    const input = await sufNode.readConsole();
    if (/^e$|^end$|^$/i.test(input)) {
        if (input) {
            out.push(input);
        }
        return out;
    } else {
        if (input) {
            out.push(input);
        }
        return await getBadges(out);
    }
}

class State {
    constructor(PACKAGE, CONFIG, configPath, canCreateOrUpdateConfig = false){
        this.PACKAGE = PACKAGE;
        this.CONFIG = CONFIG;
        this.configPath = configPath;
        this.canCreateOrUpdateConfig = canCreateOrUpdateConfig;
    }
    async getConfigSection(type) {
        if (this.CONFIG[type] !== undefined) {
            return this.CONFIG[type];
        } else if (this.canCreateOrUpdateConfig) {
            const config = await createOrUpdateConfig(this.PACKAGE, type, this);
            if (config && config[type]) {
                return config[type];
            }
        }
    }
}

const logIntro = (module)=>license.log('intro', 'SUF Cli', `(${module})`);
async function run() {
    const ARGS = getArgs();
    const Package = await getPackageJson();
    const { config , configPath  } = await getConfig(Package);
    const STATE = new State(Package, config, configPath);
    const ARG0 = ARGS[0] && ARGS[0].replace(/^--?/, '').toLowerCase();
    switch(ARG0){
        case 'a':
        case 'all':
            logIntro('All Modules');
            await CallAll(STATE);
            break;
        case 'b':
        case 'badges':
            logIntro('Badges Module');
            STATE.canCreateOrUpdateConfig = true;
            await license.Badges(STATE);
            break;
        case 't':
        case 'ts':
        case 'd.ts':
        case 'docs':
            logIntro('TsDoc Module');
            STATE.canCreateOrUpdateConfig = true;
            await license.TsDoc(STATE);
            break;
        case 'l':
        case 'license':
            logIntro('License Module');
            STATE.canCreateOrUpdateConfig = true;
            await license.License(STATE);
            break;
        case 'h':
        case 'help':
            logIntro('Help');
            license.log('help');
            break;
        default:
            logIntro('default, All Modules');
            await CallAll(STATE);
            break;
    }
}
async function CallAll(STATE) {
    await license.Badges(STATE);
    await license.TsDoc(STATE);
    await license.License(STATE);
}

run();
