'use strict';

var fs = require('fs');
var sufNode = require('suf-node');
var path = require('path');
var fetch = require('node-fetch');
var sufLog = require('suf-log');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fetch__default = /*#__PURE__*/_interopDefaultLegacy(fetch);

const colors = {
    info: '#f64',
    yellow: '#fc2',
    blue: '#2af',
    red: '#f26',
    gray: '#aaa',
    error: '#f00'
};
const genMessage = (module)=>`Generated ${module} at:`;
const loggers = {
    info: {
        styles: [
            colors.info,
            colors.yellow
        ]
    },
    intro: {
        styles: [
            colors.red,
            colors.blue
        ]
    },
    error: {
        styles: [
            {
                color: colors.error,
                'font-weight': 'bold'
            }
        ]
    },
    help: {
        styles: [
            {
                color: '#72a',
                background: '#111'
            },
            {
                color: colors.red,
                background: '#222'
            },
            {
                color: colors.blue,
                background: '#222'
            }
        ]
    }
};
function log(type, ...messages) {
    if (type !== 'help') {
        sufLog.Log(...messages.map((msg, i)=>sufLog.styler(msg, loggers[type].styles[i])));
    } else {
        console.log(nodeHelpMessage({
            text: `    INFO: All arguments can start with - or --, but i would recommend to just use letters.
    a | all: Calls all modules.
    b | badges: Calls the badges module.
    t | ts | d.ts | docs: Calls the tsDoc module.
    l | licence: Calls the license module.
    h | help: Displays this Message.`,
            splitter: ':',
            firstColumnWidth: 25,
            secondColumnWidth: 90
        }, {
            styles: loggers.help,
            rawMessages: []
        }));
    }
}
const nodeHelpMessage = (preset, data)=>{
    if (preset.text) {
        const lines = preset.text.split(/\n/g);
        preset.text = undefined;
        let output = '';
        output += nodeHelpMessage(preset, {
            ...data,
            rawMessages: [
                '\n ',
                ' '
            ]
        });
        for (const line of lines){
            output += nodeHelpMessage(preset, {
                ...data,
                rawMessages: '\n'.concat(line).split(preset.splitter)
            });
        }
        output += nodeHelpMessage(preset, {
            ...data,
            rawMessages: [
                '\n ',
                ''
            ]
        });
        return output.replace(/^\n/, '');
    } else {
        const messages = [];
        for(let i = 0; i < data.rawMessages.length; i++){
            messages.push({
                message: data.rawMessages[i]
            });
        }
        const lastIndex = messages.length - 1;
        /*istanbul ignore else */ if (messages.length > 1) {
            const space = preset.firstColumnWidth - messages[0].message.length;
            /*istanbul ignore else */ if (space >= 0) {
                messages[0].message = messages[0].message.replace(/^(\n)?/, '$1 ').concat(' '.repeat(space));
            }
            let concatenatedMessages = '';
            for(let i1 = 1; i1 < messages.length; i1++){
                concatenatedMessages += messages[i1].message;
            }
            const endingSpace = preset.secondColumnWidth - (concatenatedMessages.length + (messages.length - 1) * 2);
            /*istanbul ignore else */ if (endingSpace >= 0) {
                messages[lastIndex].message = messages[lastIndex].message.concat(' '.repeat(endingSpace));
            }
        }
        let output1 = '';
        for(let i2 = 0; i2 < messages.length; i2++){
            messages[i2].message = messages[i2].message.replace(/^(\n)?/, '$1 ');
            output1 += sufLog.styler(messages[i2].message.concat('  '), data.styles[i2]);
        }
        return output1;
    }
};

const insertionMarker = {
    marker: 'SUF_CLI_DOC_INSERTION_MARKER',
    regex: /SUF_CLI_DOC_INSERTION_MARKER/g
};
const Markers = {
    tsDoc: {
        regex: /<span id="DOC_GENERATION_MARKER_0"><\/span>[\S\s]*<span id="DOC_GENERATION_MARKER_1"><\/span>/,
        marker: '<span id="DOC_GENERATION_MARKER_0"></span>\n__INSERT_GENERATED__\n_Generated with_ **[suf-cli](https://www.npmjs.com/package/suf-cli)**\n<span id="DOC_GENERATION_MARKER_1"></span>'
    },
    badges: {
        regex: /<span id="BADGE_GENERATION_MARKER_0"><\/span>[\S\s]*<span id="BADGE_GENERATION_MARKER_1"><\/span>/,
        marker: '<span id="BADGE_GENERATION_MARKER_0"></span>\n__INSERT_GENERATED__\n<span id="BADGE_GENERATION_MARKER_1"></span>'
    },
    license: {
        regex: /<span id="LICENSE_GENERATION_MARKER_0"><\/span>[\S\s]*<span id="LICENSE_GENERATION_MARKER_1"><\/span>/,
        marker: '<span id="LICENSE_GENERATION_MARKER_0"></span>\n__INSERT_GENERATED__\n<span id="LICENSE_GENERATION_MARKER_1"></span>'
    }
};
function readFileAndAddMarker(file, type) {
    let output = insertionMarker.marker;
    if (fs.existsSync(file)) {
        const inputFile = fs.readFileSync(file).toString();
        output = inputFile.replace(Markers[type].regex, insertionMarker.marker);
        if (!insertionMarker.regex.test(output)) {
            output += `\n${insertionMarker.marker}`;
        }
    }
    return output;
}
function insertGenerated(generatedText, type) {
    return Markers[type].marker.replace(/__INSERT_GENERATED__/, generatedText);
}

async function Badges(STATE) {
    const CONFIG = await STATE.getConfigSection('badges');
    if (!CONFIG) return;
    const outPath = CONFIG.out || 'README.md';
    const generatedBadges = genBadges(CONFIG);
    const input = readFileAndAddMarker(outPath, 'badges');
    await fs.promises.writeFile(outPath, input.replace(insertionMarker.regex, insertGenerated(generatedBadges, 'badges')));
    log('info', genMessage('Badges'), outPath);
}
function genBadges(CONFIG) {
    let out = '';
    for (const _badge of CONFIG.badges){
        out += addBadge(_badge, CONFIG);
    }
    return out.replace(/^ /, '');
}
function addBadge(_badge, CONFIG) {
    const badge = parseBadge(_badge);
    if (Array.isArray(badge)) {
        log('info', `Badge: ${badge} is an invalid badge type.`);
        return '';
    }
    const { type , link , params , split , customBadge , customLink  } = badge;
    let baseLink = 'https://img.shields.io';
    switch(type){
        case 'install':
        case 'npmDep':
        case 'npmNode':
        case 'npmLicense':
            baseLink = 'https://badgen.net';
            break;
        case 'codecov':
            baseLink = 'https://codecov.io';
            break;
    }
    if (BADGES[type] || customBadge && LINKS[link] || customLink) {
        return ` [![${customBadge ? 'Custom' : type}](${customBadge ? '' : baseLink}${customBadge ? customBadge : replacePlaceholders(CONFIG, BADGES[type])}${params ? params : ''})](${customLink ? customLink : replacePlaceholders(CONFIG, LINKS[link])})`;
    }
    log('info', `Badge: "${split.join(' ')}" is not valid.`);
    return '';
}
function parseBadge(badge) {
    if (typeof badge === 'string') {
        let customLink = '';
        if (/ link=.+/.test(badge)) {
            customLink = badge.replace(/.*?link=([^ ]*) ?.*/, '$1');
        }
        let customBadge = '';
        if (/badge=.+/.test(badge)) {
            customBadge = badge.replace(/badge=([^ ]*) ?.*/, '$1');
        }
        const split = badge.split(' ');
        const type = split[0];
        const link = split[1];
        const params = split[2];
        return {
            type,
            customBadge,
            link,
            customLink,
            params,
            split
        };
    }
    if (typeof badge === 'object' && Array.isArray(badge)) {
        return {
            type: badge[0],
            link: badge[1],
            params: badge[2] || '',
            customBadge: '',
            customLink: '',
            split: badge
        };
    }
    return [
        badge
    ];
}
function replacePlaceholders(CONFIG, text) {
    text = text.replace(/<NAME>/g, CONFIG.name);
    text = text.replace(/<GITHUB>/g, CONFIG.github);
    text = text.replace(/<VSCODE>/g, CONFIG.vscode || '');
    text = text.replace(/<REPO>/g, CONFIG.repo);
    return text;
}
const BADGES = {
    /** circleCi build. */ circleci: '/circleci/build/github/<GITHUB>/<REPO>',
    /** codecov percentage. */ codecov: '/gh/<GITHUB>/<REPO>/branch/master/graph/badge.svg',
    /** Vscode Extension Version. */ vscV: '/visual-studio-marketplace/v/<VSCODE>',
    /** Vscode Extension downloads. */ vscD: '/visual-studio-marketplace/d/<VSCODE>',
    /** Vscode Extension installs. */ vscI: '/visual-studio-marketplace/i/<VSCODE>',
    /** Vscode Extension ratings. */ vscR: '/visual-studio-marketplace/r/<VSCODE>',
    /** Bundlephobia Min. */ min: '/bundlephobia/min/<NAME>',
    /** Bundlephobia Minzip. */ minzip: '/bundlephobia/minzip/<NAME>',
    /** Packagephobia Install. */ install: '/packagephobia/install/<NAME>',
    /** Packagephobia Publish. */ publish: '/packagephobia/publish/<NAME>',
    /** Npm Version. */ npmV: '/npm/v/<NAME>',
    /** Npm Weekly Downloads. */ npmDW: '/npm/dw/<NAME>',
    /** Npm Monthly Downloads. */ npmDM: '/npm/dm/<NAME>',
    /** Npm Yearly Downloads. */ npmDY: '/npm/dy/<NAME>',
    /** Npm Total Downloads. */ npmDT: '/npm/dt/<NAME>',
    /** Npm Types. */ npmTypes: '/npm/types/<NAME>',
    /** Npm License. */ npmLicense: '/npm/license/<NAME>',
    /** Npm Node. */ npmNode: '/npm/node/<NAME>',
    /** Npm Dependents. */ npmDep: '/npm/dependents/<NAME>',
    /** GitHub Followers. */ githubFollowers: '/github/followers/<GITHUB>',
    /** GitHub Forks. */ githubForks: '/github/forks/<GITHUB>/<REPO>',
    /** GitHub Starts. */ githubStars: '/github/stars/<GITHUB>/<REPO>',
    /** GitHub Issues. */ githubIssues: '/github/issues/<GITHUB>/<REPO>',
    /** GitHub Last Commit. */ githubLastCommit: '/github/last-commit/<GITHUB>/<REPO>'
};
// https://app.circleci.com/pipelines/github/TheRealSyler/sass-formatter
const LINKS = {
    /** Npm package. */ npm: 'https://www.npmjs.com/package/<NAME>',
    /** Github Repo. */ github: 'https://github.com/<GITHUB>/<REPO>',
    /** circleCi Repo Pipelines. */ circleci: 'https://app.circleci.com/pipelines/github/<GITHUB>/<REPO>',
    /** Visual Studio marketplace. */ vscode: 'https://marketplace.visualstudio.com/items?itemName=<VSCODE>',
    /** Bundlephobia Link. */ bundle: 'https://bundlephobia.com/result?p=<NAME>',
    /** Packagephobia Link. */ package: 'https://packagephobia.now.sh/result?p=<NAME>',
    /** codecov Link. */ codecov: 'https://codecov.io/gh/<GITHUB>/<REPO>'
};

const codeBlock = '```';
async function TsDoc(STATE) {
    const section = await STATE.getConfigSection('tsDoc');
    if (section === undefined) return;
    const CONFIG = addDefaultsToConfig(section);
    const filesPaths = await getPaths(CONFIG);
    const input = readFileAndAddMarker(CONFIG.out, 'tsDoc');
    const declarationBlockRegex = /(\/\*\*?[\S\s]*?\*\/\n)?export[ \t]*(declare|interface)[ \t]*([\w-]*)[ \t]*([\w-]*)(.*?;|[\S\s]*?^})/gm;
    let output = '';
    const navLinks = [];
    for (const path of filesPaths){
        const fileText = fs.readFileSync(path).toString();
        const fileName = path.replace(/.*(\/|\\\\|\\)([\w\.-]*)\.d\.ts/, '$2');
        let match;
        navLinks.push(`#${fileName}`);
        let fileOutput = `\n### ${fileName}\n`;
        while((match = declarationBlockRegex.exec(fileText)) !== null){
            /* istanbul ignore if */ if (match.index === declarationBlockRegex.lastIndex) {
                declarationBlockRegex.lastIndex++;
            }
            let [all, comment, declaration, type, name, content] = match;
            const isInternalDeclaration = /^internal/i.test(convertComment(comment));
            if (!isInternalDeclaration) {
                navLinks.push(declaration === 'interface' ? type : name);
                fileOutput += `\n##### ${declaration === 'interface' ? type : name}\n
${codeBlock}ts
${all.replace(/export ?| ?declare ?/g, '').replace(/\n^[ \t]*private .*$/gm, '')}
${codeBlock}
`;
            }
        }
        output += fileOutput;
    }
    const title = CONFIG.title ? `\n# ${CONFIG.title}\n` : '';
    await fs.promises.writeFile(CONFIG.out, input.replace(insertionMarker.regex, insertGenerated(`${title}${createNav(navLinks)}${output}`, 'tsDoc')));
    log('info', genMessage('TsDoc'), CONFIG.out);
}
function addDefaultsToConfig(config) {
    return {
        ...config,
        dir: /* istanbul ignore next */ config?.dir || '',
        out: /* istanbul ignore next */ config?.out || 'README.md'
    };
}
async function getPaths(CONFIG) {
    const dir = await sufNode.Walk(path.resolve('./', CONFIG.dir));
    if (CONFIG.exclude !== undefined && CONFIG.include !== undefined) {
        log('error', '[suf-cli:tsDoc] Cannot use option "include" and "exclude" at the same time, all d.ts files will be used.');
        return dir.filter((fileName)=>fileName.endsWith('d.ts'));
    }
    const isInclude = CONFIG.include !== undefined ? CONFIG.include.length === 0 : true;
    const type = isInclude ? 'exclude' : 'include';
    const checkArr = CONFIG[type] || [];
    const includeCheck = (a, b)=>a === b;
    const excludeCheck = (a, b)=>a !== b;
    const check = isInclude ? includeCheck : excludeCheck;
    const filename = (name)=>path.basename(name).replace(/\.d\.ts$/, '');
    const filesPaths = dir.filter((fileName)=>fileName.endsWith('d.ts') && check(checkArr.indexOf(filename(fileName)), -1));
    return filesPaths;
}
function createNav(links) {
    let output = '';
    for (const link of links){
        if (link.startsWith('#')) {
            output += `\n- **[${link.replace(/#/, '')}](${link.toLowerCase()})**\n\n`;
        } else {
            output += `  - [${link}](#${link.toLowerCase()})\n`;
        }
    }
    return output;
}
function convertComment(comment) {
    return comment ? comment.replace(/(\/?\*\*?\/?|[\t \n]*)/g, '') : '';
}

async function License(STATE) {
    const CONFIG = await STATE.getConfigSection('license');
    if (!CONFIG) return;
    const license = await (await fetch__default["default"](`https://api.github.com/licenses/${CONFIG.type}`)).json();
    const OUT = CONFIG.out || 'README.md';
    const FILE = CONFIG.file || 'LICENSE';
    const YEAR = CONFIG.year ? CONFIG.year : new Date().getFullYear();
    const input = readFileAndAddMarker(OUT, 'license');
    const readmeText = `Copyright (c) YEAR NAME Licensed under the TYPE license.`.replace(/NAME/, CONFIG.name).replace(/YEAR/, YEAR.toString()).replace(/TYPE/, license.spdx_id);
    await fs.promises.writeFile(OUT, input.replace(insertionMarker.regex, insertGenerated(readmeText, 'license')));
    log('info', genMessage('License'), OUT);
    await fs.promises.writeFile(FILE, license.body.replace(/\[year\]/, YEAR).replace(/\[fullname\]/, CONFIG.name));
    log('info', genMessage('License'), FILE);
}

exports.Badges = Badges;
exports.License = License;
exports.TsDoc = TsDoc;
exports.colors = colors;
exports.log = log;
